/*
 * Copyright 2001-2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.  
 */
package org.apache.ws.eventing;

import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.pubsub.NotificationConsumer;
import org.apache.ws.pubsub.NotificationProducer;
import org.apache.ws.pubsub.SubscriptionEndConsumer;
import org.apache.ws.resource.ResourceHome;
import org.apache.ws.resource.ResourceKey;
import org.apache.ws.resource.lifetime.ResourceTerminationListener;

import java.util.Calendar;

public abstract class AbstractSubscription implements org.apache.ws.pubsub.Subscription, org.apache.ws.resource.Resource {
    
    protected NotificationConsumer consumer;
    protected SubscriptionEndConsumer endconsumer;

    protected String m_id;
    protected Calendar m_terminationTime;     
    protected Calendar m_creationTime;  
    protected ResourceHome rH;
    protected ResourceKey rK; 
    protected boolean m_usenotify;
    /** The EndpointReference for this resource **/
    protected EndpointReference m_endpointReference;
    
    public void setResourceKey(ResourceKey rK){
        this.rK=rK;
    }
    public ResourceKey getResourceKey(){
        return(this.rK);
    }
    public void setResourceHome(ResourceHome rH){
        this.rH=rH;
    }
    public void setEpr(EndpointReference epr){
        this.m_endpointReference=epr;
    }
    
    public Object getID(){
        return this.m_id;
    }
    public void setID(Object id){
        this.m_id=(String)id;
    }
    
    public void setNotificationConsumer(NotificationConsumer consumer) {
        this.consumer = consumer;
    }
    
    public void setSubscriptionEndConsumer(SubscriptionEndConsumer endconsumer) {
        this.endconsumer = endconsumer;
    }
     
    public NotificationConsumer getNotificationConsumer() {
        return this.consumer;
    }
    public SubscriptionEndConsumer getSubscriptionEndConsumer() {
        return this.endconsumer;
    }
    public NotificationProducer getNotificationProducer() {
        return null;
    }
    public java.util.Calendar getCreationTime() {
        return m_creationTime;
    }
    
    public boolean getUseNotify(){
        return m_usenotify;
    }
    public org.apache.ws.pubsub.Filter getFilters(){
        return null;
    }
    
    public java.util.Calendar getTerminationTime(){
        return m_terminationTime;
    }     
    public void addTerminationListener(ResourceTerminationListener resourceTerminationListener)
    {
        //this is a no op  todo Stefan I need to know if you want to leverege this
    }
    /**
     * Returns the EndpointReference associated with this Resource.
     *
     * @return The Resource's EndpointReference or null if the EndpointReference has not been set.
     *
     * Note: It is the responsibility of the Resource creator to set the EndpointReference (i.e. ResourceHome impl)
     */
    public org.apache.ws.addressing.EndpointReference getEndpointReference()
    {
        return m_endpointReference;
    }

}
