/*
 * Copyright 2001-2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ws.eventing;

import edu.berkeley.cs.db.yfilter.filter.EXfilterBasic;
import edu.berkeley.cs.db.yfilter.filter.SystemGlobals;
import edu.berkeley.cs.db.yfilterplus.queryparser.Query;
import edu.berkeley.cs.db.yfilterplus.queryparser.XPQuery;
import edu.berkeley.cs.db.yfilterplus.xmltree.XMLTree;
import org.apache.axis.message.addressing.Constants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.pubsub.Filter;
import org.apache.ws.resource.InvalidResourceKeyException;
import org.apache.ws.resource.JndiConstants;
import org.apache.ws.resource.RemoveCallback;
import org.apache.ws.resource.RemoveNotSupportedException;
import org.apache.ws.resource.ResourceException;
import org.apache.ws.resource.ResourceKey;
import org.apache.ws.resource.ResourceUnknownException;
import org.apache.ws.resource.impl.SimpleTypeResourceKey;

import javax.xml.namespace.QName;
import java.util.Calendar;

public class SubscriptionHome extends org.apache.ws.resource.impl.AbstractResourceHome{
    private static final Log LOG = LogFactory.getLog( SubscriptionHome.class.getName() );
    /**  The service endpoint name as registered with the SOAP Platform.  This is useful for building EPR's. **/
    public static final javax.xml.namespace.QName SERVICE_NAME = javax.xml.namespace.QName.valueOf("{http://ws.apache.org/notification/base/service/SubscriptionManager}SubscriptionManager");


    /** The management PortType associated with this resource. This is useful for building EPR's.**/
    public static final javax.xml.namespace.QName PORT_TYPE = javax.xml.namespace.QName.valueOf("http://ws.apache.org/notification/base/service/SubscriptionManager}SubscriptionManagerPortType");

    /** The WSDL Port name associated with the resource. This is useful for building EPR's. **/
    public static final String PORT_NAME = "SubscriptionManager";

    /** The name of the resource key for this resource. **/
    public static final javax.xml.namespace.QName RESOURCE_KEY_NAME = javax.xml.namespace.QName.valueOf("{http://ws.apache.org/notification/base/service/SubscriptionManager}ResourceID");


    public static final String HOME_LOCATION = JndiConstants.CONTEXT_NAME_SERVICES + "/" + SERVICE_NAME.getLocalPart() + "/" + JndiConstants.ATOMIC_NAME_HOME;

    public int c=0;

//    private java.util.HashMap subs;
    private java.util.HashMap queriesBack;
    private String[] queries;

    EXfilterBasic yfilter;

    /** Creates a new instance of ResourceHome */
    public SubscriptionHome() {
        yfilter = new EXfilterBasic();
 //       subs=  new java.util.HashMap();
        queriesBack=  new java.util.HashMap();
        queries=  new String[1000000];
        LOG.info("ResourceHome"+this.toString());
    }

    public org.apache.ws.resource.Resource getInstance(org.apache.ws.resource.ResourceContext resourceContext) throws org.apache.ws.resource.ResourceException, org.apache.ws.resource.ResourceContextException, org.apache.ws.resource.ResourceUnknownException {
        LOG.info("getInstance ResourceHome");
        return find( resourceContext.getResourceKey() );
    }

    public Subscription create(Filter filter, Calendar expires, java.net.URI delivery){
        Subscription rs = new Subscription(filter,expires, delivery);
        rs.setResourceHome(this);
        String id = (String) rs.getID();

        SimpleTypeResourceKey resourceKey = new SimpleTypeResourceKey( QName.valueOf( getResourceKeyName()), id);
        //TODO !!!!! woher kommt publisher epr url ??? HACK for tcpmon 12345-8080
        String subscriptionAddress = "http://localhost:12345/axis/services/SubscriptionManager";
        EndpointReference subscriptionEPR = getEndpointReference( subscriptionAddress, resourceKey, Constants.NS_URI_ADDRESSING_2004_08 );
        rs.setEpr(subscriptionEPR);
        rs.setResourceKey(resourceKey);

        addSubscription(rs);
        return rs;
    }

/*
 *-------------------------------------------------------------------
 */

    private void addSubscription(Subscription s){
        try{
            c++;
            Query query = XPQuery.parseQuery((String)s.getFilter().getExpression(),c);
            LOG.info("addSubscription "+s.getID()+" query :"+query);
            yfilter.addQuery(query);

            add(s.getResourceKey(),s);
            queriesBack.put(s.getID(),query);
            queries[c]=(String)s.getID();
        }catch(Exception e){
            e.printStackTrace();
        }
    }
    /*
    public Resource find( ResourceKey key ) throws ResourceException {
        LOG.info("find "+ key.getValue()+" have "+subs.size());
        if(subs.containsKey(key.getValue())){
            RemoteSubscription s = (RemoteSubscription)subs.get(key.getValue());
            return s;
        }else
            throw new ResourceUnknownException();
    }
    */
    /**
     * Removes a resource. If the resource implements the {@link RemoveCallback RemoveCallback} interface, the
     * implementation must invoke the remove() operation on the resource itself.
     *
     * @throws ResourceUnknownException    if no resource exists with the given key
     * @throws InvalidResourceKeyException if the resource key is invalid.
     * @throws RemoveNotSupportedException if remove operation is not supported.
     * @throws ResourceException           if any other error occurs.
     */
    public void remove( ResourceKey key ) throws ResourceException {
        LOG.info("removeSubscription : "+key.getValue());
        Query q=(Query)queriesBack.get(key.getValue());
        yfilter.deleteQuery(q,q.getQueryId());
        //TODO remove from Hashmap
        super.remove(key);
    }

    public void notify(Object message) {
        //first check dialect
        try{
            org.apache.axis.message.SOAPEnvelope env = (org.apache.axis.message.SOAPEnvelope)message;
            String xml = org.apache.axis.utils.XMLUtils.ElementToString(env.getBody());
            LOG.info("Notify: ");
            LOG.info("--"+xml+"--");
            //copy

            //first parse and drive YFilter,
            XMLTree tree = new XMLTree(new java.io.StringReader(xml));
            yfilter.setEventSequence(tree.getEvents());
            yfilter.startParsing();

            // print the matched queries //
            if (SystemGlobals.hasQueries) {
                yfilter.printQueryResults(System.out);
            }else{
                LOG.info("no match");
            }
            //get matches
            java.util.Iterator it= yfilter.getMatchedQueries().iterator();
            while(it.hasNext()){
                Integer qid = (Integer) it.next();
                String sid = queries[qid.intValue()];
                SimpleTypeResourceKey rk = new SimpleTypeResourceKey( QName.valueOf( getResourceKeyName()), sid);
                Subscription s = (Subscription)super.find(rk);
                //notify matches
                LOG.info("notify : "+sid);
                s.getNotificationConsumer().notify(s,env.getBody());
            }
            yfilter.clear();

        }catch(Exception e){
            e.printStackTrace();
        }
    }

    public QName getServiceName()
    {
        return SERVICE_NAME;
    }

    public QName getPortType()
    {
        return PORT_TYPE;
    }

    public String getServicePortName()
    {
        return PORT_NAME;
    }

    public QName getResourceKeyNameQName()
    {
        return RESOURCE_KEY_NAME;
    }

}
