/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base.impl;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.resource.properties.query.impl.XmlBeansQueryExpression;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.ws.util.i18n.Messages;
import org.apache.ws.util.xml.impl.XmlBeansNamespaceContext;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.TopicExpressionType;

import javax.xml.rpc.JAXRPCException;
import java.net.URI;

/**
 * Impl of TopicExpression whose constructor takes an XMLBean.
 *
 * @author Ian Springer
 */
public class XmlBeansTopicExpression implements TopicExpression, XmlObjectWrapper
{

    private static final Log LOG = LogFactory.getLog( XmlBeansQueryExpression.class );
    public static final Messages MSG = MessagesImpl.getInstance();

    private XmlObject m_queryExprXBean;
    private URI m_dialect;
    private Object m_content;
    private Object m_nsContext;

    /**
     * Creates a new {@link XmlBeansTopicExpression} object.
     *
     * @param topicExprXBean DOCUMENT_ME
     *
     * @throws JAXRPCException          DOCUMENT_ME
     */
    public XmlBeansTopicExpression( TopicExpressionType topicExprXBean )
    {
        this( topicExprXBean, getDialect( topicExprXBean ) );
    }

    protected XmlBeansTopicExpression( XmlObject xBean, URI dialect )
    {
        m_queryExprXBean = xBean;
        m_dialect = dialect;
        XmlObject[] childElems = XmlBeanUtils.getChildElements( m_queryExprXBean );
        if ( childElems.length > 1 )
        {
            throw new JAXRPCException( MSG.getMessage( Keys.QUERY_ONLY_ONE_NODE ) );
        }
        if ( childElems.length == 1 )
        {
            m_content = childElems[0];
        }
        else
        {
            m_content = XmlBeanUtils.getValue( m_queryExprXBean );
        }
        m_nsContext = new XmlBeansNamespaceContext( m_queryExprXBean );
        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( MSG.getMessage( Keys.QUERY_EXPR, toString() ) );
        }
    }

    private static URI getDialect(
            TopicExpressionType topicExprElem )
    {
        try
        {
            return new URI( topicExprElem.getDialect() );
        }
        catch ( Exception e )
        {
            throw new JAXRPCException( MSG.getMessage( Keys.FAILED_INIT_DIALECT, e ) );
        }
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public Object getContent()
    {
        return m_content;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public URI getDialect()
    {
        return m_dialect;
    }

    public Object getNamespaceContext()
    {
        return m_nsContext;
    }

    public void setNamespaceContext( Object nsContext )
    {
        m_nsContext = nsContext;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public String toString()
    {
        return new ToStringBuilder( this ).append( MSG.getMessage( Keys.DIALECT ), m_dialect ).append(
                MSG.getMessage( Keys.CONTENT ), m_content )
                .toString();
    }

    public XmlObject getXmlObject()
    {
        return m_queryExprXBean;
    }

}
