/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base.v2004_06.impl;

import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.notification.base.Subscription;
import org.apache.ws.notification.base.WsnNamespaceVersionHolder;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.resource.JndiConstants;
import org.apache.ws.resource.Resource;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.ResourceContextException;
import org.apache.ws.resource.ResourceException;
import org.apache.ws.resource.ResourceKey;
import org.apache.ws.resource.ResourceUnknownException;
import org.apache.ws.resource.impl.AbstractResourceHome;
import org.apache.ws.resource.impl.SimpleTypeResourceKey;
import org.apache.ws.resource.properties.query.QueryExpression;

import javax.xml.namespace.QName;
import java.lang.reflect.Constructor;
import java.util.Calendar;

/**
 * TODO
 */
public class SubscriptionHome extends AbstractResourceHome
{

    /**
     * The service endpoint name as registered with the SOAP Platform.  This is useful for building EPR's. *
     */
    public static final javax.xml.namespace.QName SERVICE_NAME = javax.xml.namespace.QName.valueOf(
            "{http://ws.apache.org/notification/base/service/SubscriptionManager}SubscriptionManager" );

    /**
     * The management PortType associated with this resource. This is useful for building EPR's.*
     */
    public static final javax.xml.namespace.QName PORT_TYPE = javax.xml.namespace.QName.valueOf(
            "{http://ws.apache.org/notification/base/service/SubscriptionManager}SubscriptionManagerPortType" );

    /**
     * The WSDL Port name associated with the resource. This is useful for building EPR's. *
     */
    public static final String PORT_NAME = "SubscriptionManager";

    /**
     * The name of the resource key for this resource. *
     */
    public static final javax.xml.namespace.QName RESOURCE_KEY_NAME = javax.xml.namespace.QName.valueOf(
            "{http://ws.apache.org/notification/base/service/SubscriptionManager}ResourceID" );

    public static final String HOME_LOCATION = JndiConstants.CONTEXT_NAME_SERVICES + "/" + SERVICE_NAME.getLocalPart() +
            "/" +
            JndiConstants.ATOMIC_NAME_HOME;

    /**
     * @param resourceContext
     *
     * @return
     *
     * @throws ResourceException
     * @throws ResourceContextException
     * @throws ResourceUnknownException
     */
    public Resource getInstance( ResourceContext resourceContext ) throws ResourceException, ResourceContextException,
            ResourceUnknownException
    {
        return find( resourceContext.getResourceKey() );
    }

    public Subscription create( Class subscriptionResourceClass,
                                EndpointReference consumerReference,
                                EndpointReference producerReference,
                                Calendar initialTerminationTime,
                                Object policy,
                                QueryExpression precondition,
                                QueryExpression selector,
                                ResourceKey producerKey,
                                String producerHomeLocation,
                                TopicExpression topicExpression,
                                boolean useNotify, WsnNamespaceVersionHolder nsSet ) throws ResourceException
    {
        try
        {
            Constructor constructor = subscriptionResourceClass.getConstructor( new Class[]{
                EndpointReference.class,
                EndpointReference.class,
                ResourceKey.class,
                String.class,
                TopicExpression.class} );
            Subscription subscription = (Subscription) constructor.newInstance(
                    new Object[]{consumerReference, producerReference, producerKey, producerHomeLocation,
                                 topicExpression} );
            subscription.setTerminationTime( initialTerminationTime );
            subscription.setUseNotify( useNotify );
            subscription.setSelector( selector );
            subscription.setPrecondition( precondition );
            subscription.setPolicy( policy );
            subscription.init();  // initialize resource properties
            String id = (String) subscription.getID();
            SimpleTypeResourceKey resourceKey = new SimpleTypeResourceKey( QName.valueOf( getResourceKeyName() ), id );
            EndpointReference subscriptionEPR = getEndpointReference( getSubscriptionAddress( producerReference ),
                    resourceKey, nsSet.getAddressingNamespace() );
            subscription.setEndpointReference( subscriptionEPR );
            add( resourceKey, subscription );
            return subscription;
        }
        catch ( Exception e )
        {
            throw new ResourceException( "Failed to create Subscription resource.", e );
        }
    }

    private String getSubscriptionAddress( EndpointReference producerReference )
    {
        String producerAddress = producerReference.getAddress();
        return producerAddress.substring( 0, producerAddress.lastIndexOf( "/" ) + 1 ) + SERVICE_NAME.getLocalPart();
    }

    public QName getServiceName()
    {
        return SERVICE_NAME;
    }

    public QName getPortType()
    {
        return PORT_TYPE;
    }

    public String getServicePortName()
    {
        return PORT_NAME;
    }

    public QName getResourceKeyNameQName()
    {
        return RESOURCE_KEY_NAME;
    }
}
