/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base.v2004_06.porttype.impl;

import org.apache.ws.notification.base.Subscription;
import org.apache.ws.notification.base.WsnNamespaceVersionHolder;
import org.apache.ws.notification.base.faults.PauseFailedFaultException;
import org.apache.ws.notification.base.faults.ResumeFailedFaultException;
import org.apache.ws.notification.base.v2004_06.impl.WsnNamespaceVersionHolderImpl;
import org.apache.ws.notification.base.v2004_06.porttype.SubscriptionManagerPortType;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.ws.resource.properties.impl.AbstractResourcePropertiesPortType;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.PauseSubscriptionDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.PauseSubscriptionResponseDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.ResumeSubscriptionDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.ResumeSubscriptionResponseDocument;

/**
 * An implementation of the WS-BaseNotification SubscriptionManager portType.
 *
 * @author Ian Springer (ian DOT springer AT hp DOT com)
 */
public class SubscriptionManagerPortTypeImpl extends AbstractResourcePropertiesPortType
        implements SubscriptionManagerPortType
{

    private static final WsnNamespaceVersionHolder NAMESPACE_SET = new WsnNamespaceVersionHolderImpl();

    public SubscriptionManagerPortTypeImpl( ResourceContext resourceContext )
    {
        super( resourceContext );
    }

    public PauseSubscriptionResponseDocument pauseSubscription( PauseSubscriptionDocument requestDoc )
    {
        Subscription subscription = (Subscription) getResource();
        try
        {
            subscription.pause();
        }
        catch ( Exception e )
        {
            throw new PauseFailedFaultException( NAMESPACE_SET,
                    "Pause failed on subscription: " + subscription.getID() );
        }
        return createPauseSubscriptionResponseDocument();
    }

    public ResumeSubscriptionResponseDocument resumeSubscription( ResumeSubscriptionDocument requestDoc )
    {
        Subscription subscription = (Subscription) getResource();
        try
        {
            subscription.resume();
        }
        catch ( Exception e )
        {
            throw new ResumeFailedFaultException( NAMESPACE_SET,
                    "Resume failed on subscription: " + subscription.getID() );
        }
        return createResumeSubscriptionResponseDocument();
    }

    private PauseSubscriptionResponseDocument createPauseSubscriptionResponseDocument()
    {
        PauseSubscriptionResponseDocument responseDoc =
                PauseSubscriptionResponseDocument.Factory.newInstance();
        responseDoc.addNewPauseSubscriptionResponse();
        return responseDoc;
    }

    private ResumeSubscriptionResponseDocument createResumeSubscriptionResponseDocument()
    {
        ResumeSubscriptionResponseDocument responseDoc =
                ResumeSubscriptionResponseDocument.Factory.newInstance();
        responseDoc.addNewResumeSubscriptionResponse();
        return responseDoc;
    }

    protected NamespaceVersionHolder getNamespaceSet()
    {
        return NAMESPACE_SET;
    }

}
