/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/


package org.apache.ws.notification.pubsub;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.SubscribeResponseDocument;

import org.apache.ws.resource.JndiConstants;
import org.apache.ws.resource.impl.SimpleTypeResourceKey;

import javax.xml.namespace.QName;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
/**
 *
 * @author  Stefan Lischke
 */
public class SubscriptionHome extends org.apache.ws.resource.impl.AbstractResourceHome {

    /**  The service endpoint name as registered with the SOAP Platform.  This is useful for building EPR's. **/
    public static final javax.xml.namespace.QName SERVICE_NAME = javax.xml.namespace.QName.valueOf("{http://ws.apache.org/notification/base/service/NotificationPort}NotificationPort");


    /** The management PortType associated with this resource. This is useful for building EPR's.**/
    public static final javax.xml.namespace.QName PORT_TYPE = javax.xml.namespace.QName.valueOf("http://ws.apache.org/notification/base/service/NotificationPort}NotificationPortPortType");

    /** The WSDL Port name associated with the resource. This is useful for building EPR's. **/
    public static final String PORT_NAME = "NotificationPort";

    /** The name of the resource key for this resource. **/
    public static final javax.xml.namespace.QName RESOURCE_KEY_NAME = javax.xml.namespace.QName.valueOf("{http://ws.apache.org/notification/base/service/NotificationPort}ResourceID");

    public static final String HOME_LOCATION = JndiConstants.CONTEXT_NAME_SERVICES + "/" + SERVICE_NAME.getLocalPart() + "/" + JndiConstants.ATOMIC_NAME_HOME;

    private static final Log LOG = LogFactory.getLog( SubscriptionHome.class.getName() );
    
    public Subscription create(){
        Subscription s = new Subscription();
        String id = (String) s.getID();

        SimpleTypeResourceKey resourceKey = new SimpleTypeResourceKey( QName.valueOf( getResourceKeyName()), id);
        add(resourceKey,s);
        return s;
    }
    
    public org.apache.ws.resource.Resource getInstance(org.apache.ws.resource.ResourceContext resourceContext) throws org.apache.ws.resource.ResourceException, org.apache.ws.resource.ResourceContextException, org.apache.ws.resource.ResourceUnknownException {
        LOG.info("getInstance SubscriptionHome");
        return find( resourceContext.getResourceKey() );
    }

    public QName getServiceName()
    {
        return SERVICE_NAME;
    }

    public QName getPortType()
    {
        return PORT_TYPE;
    }

    public String getServicePortName()
    {
        return PORT_NAME;
    }

    public QName getResourceKeyNameQName()
    {
        return RESOURCE_KEY_NAME;
    }
}
