/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics;

import org.apache.ws.notification.topics.expression.TopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpression;

/**
 * The set of topics that a particular NotificationProducer supports.
 */
public interface TopicSpaceSet
{

    TopicSpace addTopicSpace( TopicSpace topicSpace ) throws Exception;

    void removeTopicSpace( String namespaceURI );

    TopicSpace getTopicSpace( String namespaceURI );

    TopicSpace[] getTopicSpaces();

    Topic[] evaluateTopicExpression(TopicExpression topicExpr) throws TopicExpressionException;

    /**
     * Returns a boolean that indicates if this topic set is fixed as defined by
     * the WS-BaseNotification specification.
     */
    boolean isFixed();

    void setFixed( boolean fixed );
}
