package org.apache.ws.notification.topics.impl;

import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.expression.TopicExpressionEngine;
import org.apache.ws.notification.topics.TopicSpace;
import org.apache.ws.notification.topics.TopicSpaceSet;
import org.apache.ws.notification.topics.expression.TopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.expression.TopicExpressionEngine;
import org.apache.ws.notification.topics.expression.impl.TopicExpressionEngineImpl;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

/**
 * @author Sal Campana
 */
public class TopicSpaceSetImpl implements TopicSpaceSet
{

    private static final TopicExpressionEngine TOPIC_EXPR_ENGINE =
            TopicExpressionEngineImpl.getInstance();  // TODO: look up from JNDI

    private Map m_topicSpaces = new Hashtable();
    private boolean m_fixed;

    public TopicSpaceSetImpl( boolean fixed )
    {
        m_fixed = fixed;
    }

    /**
     * Adds the TopicSpace to the TopicSpaceSet.
     *
     * Note. If the TopiSpace already exists, this method will add all the known Topics
     * to the passed-in TopicSpace and set it as the current TopicSpace in the TopicSpaceSet.
     * This will ensure there is only 1 TopicSpace in the set and it is complete...
     *
     * @param topicSpace
     */
    // TODO: throw more specific exceptions
    public TopicSpace addTopicSpace(TopicSpace topicSpace) throws Exception
    {
        String namespaceURI = topicSpace.getTargetNamespace();
        //if topicSpace exists, simply add all topics to topicspace
        if(m_topicSpaces.containsKey(namespaceURI))
        {
            TopicSpace topicSpc = (TopicSpace) m_topicSpaces.get(namespaceURI);
            Iterator iterator = topicSpc.topicIterator();
            while (iterator.hasNext())
            {
                Topic topic = (Topic) iterator.next();
                topicSpace.addTopic(topic);
            }
        }
        m_topicSpaces.put(namespaceURI, topicSpace);
        return topicSpace;
    }

    public void removeTopicSpace(String namespaceURI)
    {
        m_topicSpaces.remove(namespaceURI);
    }

    public TopicSpace getTopicSpace(String namespaceURI)
    {
        return (TopicSpace) m_topicSpaces.get(namespaceURI);
    }

    public TopicSpace[] getTopicSpaces()
    {
        return (TopicSpace[]) m_topicSpaces.values().toArray(new TopicSpace[0]);
    }

    public Topic[] evaluateTopicExpression(TopicExpression topicExpr)
            throws TopicExpressionException
    {
        return TOPIC_EXPR_ENGINE.evaluateTopicExpression( this, topicExpr );
    }

    public boolean isFixed()
    {
        return m_fixed;
    }

    public void setFixed( boolean fixed )
    {
        m_fixed = fixed;
    }
}
