/*
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.ws.pubsub;

import org.xmlsoap.schemas.ws.x2004.x08.eventing.FilterType;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
//import org.apache.xmlbeans.XmlObject;
/**
 *
 * @author  Stefan Lischke
 */

public class FilterFactory { 
    private static final Log LOG = LogFactory.getLog( FilterFactory.class.getName() );

    public static String xpathuri = "http://www.w3.org/TR/1999/REC-xpath-19991116";
    public static String topicsuri = "http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Concrete";
    public static Filter createFilter(FilterType ft){        
        String dialect = ft.getDialect();
        Filter f = null;
         
        if((dialect==null)||(dialect.equals(xpathuri))){
            //its an XPathFilter
            f= new XPathFilter(ft.newCursor().getTextValue(),dialect);
        }else if(dialect.equals(topicsuri)){
            //TODO testing if topicexpression is ok
            try{
                f= new TopicFilter(ft.newDomNode(),new java.net.URI(dialect));
            }catch(java.net.URISyntaxException e){
                //TODO
                e.printStackTrace();
            }
        }
        LOG.info("FilterFactory : "+dialect+ " FilterExpr: " +f.getExpression());        
        return f;
    }
    
}
