/*
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ws.pubsub;

import java.util.Calendar;

public interface Subscription
{
    /**
     * Cancel this subscription.
     */
    void unsubscribe();

    /**
     * Gets the termination time for this subscription (i.e. the time at which it expires).
     *
     * @return the termination time
     */
    Calendar getTerminationTime();

    /**
     * Sets the termination time for this subscription (i.e. the time at which it expires).
     *
     * @param terminationTime the termination time
     */
    void setTerminationTime( Calendar terminationTime );

    /**
     * Returns a list of any notification filters that are associated with this subscription. The filters are ordered in
     * the order in which they will be applied to notifications.
     *
     * @return a list of any notification filters that are associated with this subscription
     */
    Filter getFilters();

    /**
     * Returns the time at which this subscription was created.
     *
     * @return the time at which this subscription was created
     */
    Calendar getCreationTime();

    /**
     * Returns the delivery mode to be used for notifications sent for this subscription. If null, a delivery mode was
     * not specified, and a default delivery mode will be used.
     *
     * @return the delivery mode to be used for notifications sent for this subscription
     */
    boolean getUseNotify();

    /**
     * Get the notification consumer associated with this subscription.
     *
     * @return the notification consumer
     */
    NotificationConsumer getNotificationConsumer();

    /**
     * Get the notification producer associated with this subscription.
     *
     * @return the notification producer
     */
    NotificationProducer getNotificationProducer();

    /**
     * Get the subscriptionEnd consumer associated with this subscription.
     *
     * @return the subscriptionEnd consumer
     */
    SubscriptionEndConsumer getSubscriptionEndConsumer();
}
