/*
 * Copyright  2004 The Apache Software Foundation.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.ws.util.uuid;

/**
 * A {@link UuidGenerator} factory.
 *
 * @author Ian P. Springer <ian_springer@hp.com>
 */
public class UuidGeneratorFactory
{
        
    public static final String DEFAULT_IMPL_CLASSNAME = "org.apache.ws.util.uuid.impl.FastUuidGenerator";

    public static UuidGenerator createUUIdGenerator() throws RuntimeException
    {
        return createUUIdGenerator( DEFAULT_IMPL_CLASSNAME );
    }

    public static UuidGenerator createUUIdGenerator( Class implClass ) throws RuntimeException
    {
        try
        {
            return (UuidGenerator) implClass.newInstance();
        }
        catch ( Exception e )
        {
            throw new RuntimeException( "Failed to instantiate UUID generator " + implClass.getName() + ".", e );
        }
    }

    public static UuidGenerator createUUIdGenerator( String implClassname ) throws RuntimeException
    {
        try
        {
            return createUUIdGenerator( Class.forName( implClassname ) );
        }
        catch ( Exception e )
        {
            throw new RuntimeException( "Failed to instantiate UUID generator " + implClassname + ".", e );
        }
    }

}
