package org.apache.ws.notification.base.impl;

import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.resource.Resource;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.ResourceContextException;
import org.apache.ws.resource.ResourceException;
import org.apache.ws.resource.ResourceUnknownException;
import org.apache.ws.resource.ResourceKey;
import org.apache.ws.resource.impl.AbstractResourceHome;
import org.apache.ws.resource.impl.SimpleTypeResourceKey;
import org.apache.ws.notification.base.WsnNamespaceVersionHolder;
import org.apache.ws.notification.base.impl.UnitResource;
import org.apache.ws.notification.base.v2004_06.impl.WsnNamespaceVersionHolderImpl;

import javax.xml.namespace.QName;

/**
 * @author Sal Campana
 */
public class UnitExampleHome extends AbstractResourceHome
{
    public static final String SERVICE_NAME = "UnitHome";
    public static final String TARGET_NAMESPACE = "http://unittest";
    public static final QName SERVICE_QNAME = new QName(TARGET_NAMESPACE, SERVICE_NAME);
    public static final String TARGET_NSPREFIX = "unit";
    public static final QName PORT_TYPE = new QName(TARGET_NAMESPACE, "UnitTestExamples");
    public static final String SERVICE_PORT_NAME = SERVICE_NAME;
    public static final QName RESOURCE_KEY_NAME = new QName(TARGET_NAMESPACE, "ResourceIdentifier", TARGET_NSPREFIX);
    private Resource m_resource;
    public static final ResourceKey RESOURCE_KEY = new SimpleTypeResourceKey(RESOURCE_KEY_NAME, "unit_key");
    public WsnNamespaceVersionHolder m_namespaceSet = new WsnNamespaceVersionHolderImpl();

    public UnitExampleHome()
    {
        setResourceKeyName( SimpleTypeResourceKey.class.getName() );
        setResourceClassName( UnitResource.class.getName() );
        try
        {
            init();
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
    }

    public QName getServiceName()
    {
        return SERVICE_QNAME;
    }

    public QName getPortType()
    {
        return PORT_TYPE;
    }

    public String getServicePortName()
    {
        return SERVICE_PORT_NAME;
    }

    public QName getResourceKeyNameQName()
    {
        return RESOURCE_KEY_NAME;
    }

    public Resource getInstance(ResourceContext resourceContext) throws ResourceException, ResourceContextException, ResourceUnknownException
    {
          if(m_resource == null)
          {
            try
            {
                UnitResource myresource = new UnitResource(); //this will create the resource IF it is has default constructor
                myresource.setID("123");
                m_resource = myresource;
                myresource.init();
                //the next line will create an EPR
                EndpointReference epr = getEndpointReference(resourceContext.getBaseURL() + "/" + getServiceName().getLocalPart(), RESOURCE_KEY, m_namespaceSet.getAddressingNamespace());
                myresource.setEndpointReference(epr); //make sure to set the EPR on your new instance
                add(RESOURCE_KEY, myresource);
            }
            catch (Exception e)
            {
                throw new ResourceException(e);
            }
          }
        return m_resource;
    }
}
